/*
  stateset.c
*/

#include <stdio.h>
#include "state.h"
#include "stateset.h"

void stateset__empty(stateset *set) {
  int i;

  for(i=0; i<MAX_STATE; i++)
    (*set)[i] = 0;
}


/* Lower level functions */
/* */
/* These functions are dependent of the implementation of stateset */
int stateset__belongs_to(stateset *set, state s) {
  return(! ((*set)[s]==0));
}

void stateset__add_state(stateset *set, state s) {
  (*set)[s] = 1;  
}

void stateset__remove_state(stateset *set, state s) {
  (*set)[s] = 0;  
}


/* Higher level functions */
/* */
/* These functions depend only on the previous functions */
int  stateset__is_empty(stateset *set) {
  int i,b = 1; /* true */
  for(i=0; i<MAX_STATE; i++)
    b = b && !(stateset__belongs_to(set,i));
  return(b);
}

int  stateset__equal(stateset *s1, stateset *s2) {
	int i=0;

	while ((i<MAX_STATE) && 
				 (stateset__belongs_to(s1, i) == stateset__belongs_to(s2, i)))
		i++;

	return (! (i<MAX_STATE));
}

/*pour initialiser set  vide et y ajouter s*/
void stateset__from_state(state s, stateset *set) {
  stateset__empty(set);
  stateset__add_state(set, s);
}

void stateset__copy(stateset *s, stateset *r) {
  int i;

  stateset__empty(r);
  for (i=0; i<MAX_STATE; i++)
    if(stateset__belongs_to(s,i))
      stateset__add_state(r,i);
}

/* No hypothesis on the variables (r=s1 or r=s2) */
void stateset__intersection(stateset *s1, stateset *s2, stateset *r) {
  int i;

  for (i=0; i<MAX_STATE; i++)
    if ((stateset__belongs_to(s1,i)) && (stateset__belongs_to(s2,i)))
      stateset__add_state(r,i);
    else
      stateset__remove_state(r,i);
}

/* No hypothesis on the variables (r=s1 or r=s2) */
void stateset__reunion(stateset *s1, stateset *s2, stateset *r) {
  int i;

  for (i=0; i<MAX_STATE; i++)
    if ((stateset__belongs_to(s1,i)) || (stateset__belongs_to(s2,i)))
      stateset__add_state(r,i);
    else
      stateset__remove_state(r,i);
}

void stateset__print(stateset *s) {
  int i;

  printf("[ ");
  for(i=0; i<MAX_STATE; i++)
    if (stateset__belongs_to(s,i)) 
      printf("%d ",i);
  printf("]");
}

void stateset__full_print(stateset *s) {
  int i;

  printf("Ensemble : [ ");
  for(i=0; i<MAX_STATE; i++)
    if (stateset__belongs_to(s,i)) 
      printf("%d ",i);
  printf("]\n");
}

/* s is interpreted as the reversed binary encoding of an unsigned int */
state stateset__encode(stateset *s)
{
  unsigned int n;
  int i;

  n = 0;
  for (i=MAX_STATE-1; i>=0; i--)
    {
      n = n*2;
      if (stateset__belongs_to(s,i))
	n = n+1;
    }
  return n;
}
