/* 
   stateset.h

   Description des ensembles d'tats utiliss par l'automate. 
   Actuellement, ces ensembles d'tats sont des tableaux d'entiers,

   La convention est la suivante : les tats pour lesquels la valeur
   dans le tableau est non nulle sont dans l'ensemble, ceux
   pour lesquels la valeur est nulle n'y sont pas. 

*/

#ifndef _STATESET_H_ 
#define _STATESET_H_   1

#include "state.h"

typedef unsigned char stateset[MAX_STATE];  /*stateset correspond  un ensemble d'tat*/

void stateset__empty(stateset *set); /*ensemble d'tat vide*/

/* Low level functions */
int  stateset__belongs_to(stateset *set, state s); /*appartenance*/
void stateset__add_state(stateset *set, state s);
void stateset__remove_state(stateset *set, state s);

/* Higher level functions */
int  stateset__is_empty(stateset *set);
int  stateset__equal(stateset *s1, stateset *s2);
void stateset__from_state(state s, stateset *set);
void stateset__copy(stateset *s, stateset *r);
void stateset__intersection(stateset *s1, stateset *s2, stateset *r);
void stateset__reunion(stateset *s1, stateset *s2, stateset *r);
void stateset__print(stateset *s);
void stateset__full_print(stateset *s);
state stateset__encode(stateset *s);

#endif /* _STATESET_H_ */
