/*
  uautomaton.c
*/

#include <stdio.h>
#include "uautomaton.h"

/*crer un automate vide*/
void uautomaton__empty (uautomaton *u )
{
  state q;    /*state et letter correspondent en fait  des entiers (ce sont des alias), on aurait pu mettre "int q,l;" */
  letter l;

  u->q0 = UNDEFINED_STATE;
  stateset__empty(&(u->F));

  for(q=0;q<MAX_STATE;q++)
    {
      for(l=0;l<MAX_LETTER;l++)
	{
	  stateset__empty(&(u->delta[q][l]));
	}
    }
}


/*ajouter un tat initial*/
void uautomaton__set_initial (uautomaton *u , state s)
{
  u->q0=s;
}


/*ajouter un tat final (s'il tait final, il ne l'est plus)*/
void uautomaton__toggle_final_state (uautomaton *u , state s)
{
  if (stateset__belongs_to(&(u->F),s))
    stateset__remove_state(&(u->F),s);
  else stateset__add_state(&(u->F),s);
}


/*ajouter une transition*/
void uautomaton__add_transition (uautomaton *u , state s1, letter l, state s2)
{
  stateset__add_state(&(u->delta[s1][l]),s2);
}


/*passer la transition*/
void uautomaton__apply_transition (uautomaton *u ,stateset *S, state s, letter l)
{
  stateset__copy(&(u->delta[s][l]),S);  /*la fonction stateset__copy sert  copier un stateset dans un autre stateset*/
}


/*pour voir si dans l'ensemble des tats actifs  la fin, il y a un tat final*/
int uautomaton__contains_final_state(uautomaton *u , stateset *q)
{
  stateset r;
  stateset__intersection(&(u->F), &q, &r);
  if !(stateset__is_empty(&r))
    return 1;
  else
    return 0;
}


int uautomaton__apply_set_transition(uautomaton *u , stateset *s ,letter l, stateset *r)
{
  stateset *v;
  int i;

  while (s[i]!=0)  /* on parcourt tous les tats actifs en cours */
    {
      uautomaton__apply_transition (u, v, s[i], l);  /* pour l'tat actif s[i], on passe la transition, ce qui donne un ensemble d'tat 'stateset' contenu dans v */

      stateset__intersection(v, r, r);               /* puis on fait l'intersection entre r et s que l'on place dans r de manire  avoir dans r l'ensemble des tats
						        actifs aprs le passage de toutes les transitions s[i] */
      i++;        /*on passe  l'tatactif suivant*/
    }
  return 1;
}


void uautomaton__epsilon_cloture(uautomaton *u, stateset *q, stateset *clq)
{
  stateset E, Eprec, Esuiv;
  int x;

  stateset__copy(x,&E);
  stateset__empty(&Eprec);
 
  while (!stateset__equal(&E,&Eprec))
    {
      stateset__copy(&E,&Eprec);
      uautomaton__apply_set_transition(u, &Eprec, EPSILON, &Esuiv);
      stateset__union(&Eprec,&Esuiv,&E);
    }
  stateset__copy(&E,clq);
}
